#include "Particle.h"
#include "partemitter.h"
#include <stdio.h>

#include <windows.h>
#include <GL\Gl.h>

CParticle::CParticle(CPartEmitter* pEmitter)
{
	m_pNext = NULL;
	m_pEmitter = pEmitter;
	m_clrColor[0] = m_clrColor[1] = m_clrColor[2] = 1.0f; // weiss
	m_clrColor[3] = 0.001f;	// *NEU* alpha sehr klein whlen
	m_vPosition[0] = m_vPosition[1] = m_vPosition[2] = 0.0f;	// Ursprung
	m_vVelocity[0] = m_vVelocity[1] = m_vVelocity[2] = 0.0f;	// keine Geschwindigkeit
	m_fSize = 1.0f;
	m_fBirthday = pEmitter->m_fCurTime;
	m_fLifetime = 5.0f;
	m_bLife = true;
	m_bBorn = true;
	m_bChangeSize = true;
	m_bIsDying = false;
	m_fChangeSize=0.0f;
	m_fFadeInSpeed = 1.0f;
}


CParticle::~CParticle()
{

}

void CParticle::Set(VECTOR vPosition,VECTOR vVelocity, float clrColor[4],float clrChangeColor[4],
					 float fSize,float fChangeSize,float fLifetime,float fFadeSpeed)
{
	memcpy(m_vPosition,vPosition,sizeof(float)*3);
	memcpy(m_vVelocity,vVelocity,sizeof(float)*3);
	m_fSize = fSize; m_fChangeSize = fChangeSize;
	memcpy(m_clrColor,clrColor,sizeof(float)*4);
	m_clrColor[3] = 0.001f;	// *NEU* alpha sehr klein whlen
	memcpy(m_clrChangeColor,clrChangeColor,sizeof(float)*4);
	m_fLifetime = fLifetime; m_fFadeSpeed = fFadeSpeed;

}

void CParticle::Update()
{
	if(m_bBorn)
	{
		// Auftauchen des Partikels durch erhhen des Alpha-Wertes
		m_clrColor[3] += (m_pEmitter->m_fFrameTime*m_fFadeInSpeed);
		
		// Prozess der "Geburt" beendet, vollen Alpha-Wert 1 setzen
		if(m_clrColor[3] >= 0.99f)
		{
			m_clrColor[3] = 1.0f;
			m_bBorn = false;
		}
	}	else	if(m_bIsDying)
	{
		// Partikel verblassen lassen, durch verringen des Alpha-Werts
		m_clrColor[3] -= (m_pEmitter->m_fFrameTime *m_fFadeSpeed);

		if(m_clrColor[3] < 0)
			m_bLife = false; // Partikel wird beim nchsten Schleifendruchlauf gelscht
	} else	if(m_pEmitter->m_fCurTime - m_fBirthday > m_fLifetime)
		m_bIsDying = true;	// Startet den "Sterbeprozess"



	// neue Position anhand der Geschwindigkeit und der Zeit seit dem letzen
	// Update berechnen
	m_vPosition[0] += (m_vVelocity[0]*m_pEmitter->m_fFrameTime);
	m_vPosition[1] += (m_vVelocity[1]*m_pEmitter->m_fFrameTime);
	m_vPosition[2] += (m_vVelocity[2]*m_pEmitter->m_fFrameTime);

	//*NEU* Farbnderung
	m_clrColor[0] += m_clrChangeColor[0]*m_pEmitter->m_fFrameTime;
	m_clrColor[1] += m_clrChangeColor[1]*m_pEmitter->m_fFrameTime;
	m_clrColor[2] += m_clrChangeColor[2]*m_pEmitter->m_fFrameTime;

	
	// *NEU* Grennderung
	if(m_bChangeSize)
	{
		m_fSize += m_fChangeSize;
		if(m_fSize <0)
			m_fSize = 0;
	}
}

void CParticle::Render()
{
	glColor4fv(m_clrColor);// Farbe des Partikel setzen
	
	glBegin(GL_TRIANGLE_STRIP); // Begin der Zeichenoperation, *NEU* Trianglestrips

		// oben rechts
		glTexCoord2i(1,1); 
		glVertex3f(m_vPosition[0]+m_fSize,m_vPosition[1]+m_fSize,m_vPosition[2]);
		
		// oben links
		glTexCoord2i(0,1);
		glVertex3f(m_vPosition[0]-m_fSize,m_vPosition[1]+m_fSize,m_vPosition[2]);
		
		// unten rechts
		glTexCoord2i(1,0); 
		glVertex3f(m_vPosition[0]+m_fSize,m_vPosition[1]-m_fSize,m_vPosition[2]);
		
		// unten links
		glTexCoord2i(0,0); 
		glVertex3f(m_vPosition[0]-m_fSize,m_vPosition[1]-m_fSize,m_vPosition[2]);
	
	glEnd();	// Ende der Zeichenoperation		
}